=begin
This extension is here to make sure .getValue() returns UESURI instead of a String, but we need
to ensure backwards compatibility for String methods, so instead of just UESURI we return special
ProptUESURI which enables UESURI methods on java String
=end

class ProptUESURI < String
 
  @@initialized = false 
  @uri = nil
  
  def initialize(uri)
    @uri = Ues::Core::Commons::Uri::UESURI.new(uri)
    super(@uri.to_s)
  end

  def method_missing(meth,*args,&block) 
    if @initialized.nil?
      @uri_methods = Ues::Core::Commons::Uri::UESURI.instance_methods(false).map(&:to_s)
      @log = UESLoggerFactory.getLogger("scrip.ues.core.property.ProptUESURI")
      @initialized = true
    end
    
    if @uri_methods.include? meth.to_s()
      @log.info("Ruby-UESURI method call on ProptUESURI: "+meth.to_s()+"()")
      @uri.send(meth, *args)
    else
      @log.info("String method call on ProptUESURI: "+meth.to_s()+"()")
      super
    end 
  end
  
  def self.method_missing(meth,*args,&block) 
    if @@initialized == false
      @uri_methods = Ues::Core::Commons::Uri::UESURI.methods(false).map(&:to_s)
      @@initialized = true
    end
    
    if @uri_methods.include? meth.to_s()
      @log.info("Static Ruby-UESURI method call on ProptUESURI: "+meth.to_s()+"()")
      Ues::Core::Commons::Uri::UESURI.send(meth, *args)
    else
      @log.info("Static String method call on ProptUESURI: "+meth.to_s()+"()")
      super
    end 
  end
end


class Ues::Core::Property::UESPropertyEntryAttributes
  
  alias old_getvalue getValue
   
 
  def getValue
    value_type = self.getValueType()
    if value_type == Ues::Core::Property::UESPropertyType::REFERENCE
      res = old_getvalue
      if res.nil?
        return nil
      end
      
      svik = ProptUESURI.new(res)
      
      return svik
    elsif value_type == Ues::Core::Property::UESPropertyType::INTERTERRITORIAL_REFERENCE
      res = old_getvalue
      if res.nil?
        return nil
      end
      if res.to_s().empty?
        return nil
      end
 
      svik = ProptUESURI.new(res)
      
      return svik
    end
    
    return old_getvalue
  end
  
  alias value getValue
end